==> liquidwar6-0.0.6beta.tar.gz: liquidwar6-0.0.6beta/script/mover.scm <==
;Liquid War 6 is a unique multiplayer wargame.
;Copyright (C)  2005, 2006, 2007, 2008, 2009  Christian Mauduit <ufoot@ufoot.org>
;
;This program is free software; you can redistribute it and/or modify
;it under the terms of the GNU General Public License as published by
;the Free Software Foundation, either version 3 of the License, or
;
;(at your option) any later version.
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
;
;Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
;Contact author        : ufoot@ufoot.org

(define %lw6-sensitivity-correction 3.)

(define lw6-mover-slow-down
  (lambda (cursor sensitivity key)
    (let (
	  (value (hash-ref cursor key))
	  )
      (hash-set! cursor key (cond
			     ((> value 0)
			      (max 0 (- value (* 2.0 (/ sensitivity %lw6-sensitivity-correction)))))
			     ((< value 0)
			      (min 0 (+ value (* 2.0 (/ sensitivity %lw6-sensitivity-correction)))))
			     (#t 0)
			     )))))

(define lw6-mover-minus
  (lambda (cursor sensitivity max-cursor-speed key)
    (let (
	  (value (hash-ref cursor key))
	  )
      (hash-set! cursor key (cond
			     ((> value 0)
			      0)
			     (#t
			      (max (- 0 max-cursor-speed) (- value (/ sensitivity %lw6-sensitivity-correction))))
			     )))))

(define lw6-mover-plus
  (lambda (cursor sensitivity max-cursor-speed key)
    (let (
	  (value (hash-ref cursor key))
	  )
      (hash-set! cursor key (cond
			     ((< value 0)
			      0)
			     (#t
			      (min max-cursor-speed (+ value (/ sensitivity %lw6-sensitivity-correction))))
			     )))))

(define lw6-mover-generic
  (lambda (get-state-func sensitivity-func max-cursor-speed-func)
    (lambda (cursor)
      (let* (
	     (state (get-state-func))
	     (up (assoc-ref state "up"))
	     (down (assoc-ref state "down"))
	     (left (assoc-ref state "left"))
	     (right (assoc-ref state "right"))
	     (sensitivity (sensitivity-func))
	     (max-cursor-speed (max-cursor-speed-func))
	     (moved #f)
	    )
	(begin
	  (if up 
	      (if down
		  #t		  
		  (lw6-mover-minus cursor sensitivity max-cursor-speed "vy"))
	      (if down
		  (lw6-mover-plus cursor sensitivity max-cursor-speed "vy")
		  (lw6-mover-slow-down cursor sensitivity "vy")))
	  (if left
	      (if right
		  #t		  
		  (lw6-mover-minus cursor sensitivity max-cursor-speed "vx"))
	      (if right
		  (lw6-mover-plus cursor sensitivity max-cursor-speed "vx")
		  (lw6-mover-slow-down cursor sensitivity "vx")))
	  (hash-set! cursor "x" (+ (hash-ref cursor "x") (hash-ref cursor "vx")))
	  (hash-set! cursor "y" (+ (hash-ref cursor "y") (hash-ref cursor "vy")))
	  )))))

(define lw6-mover-mouse-func
  (lambda (cursor)
    (let* (
	   (video-mode (c-lw6gfx-get-video-mode))
	   (width (assoc-ref video-mode "width"))
	   (height (assoc-ref video-mode "height"))
	   (center-x (inexact->exact (round (/ width 2))))
	   (center-y (inexact->exact (round (/ height 2))))
	   (mouse-state (c-lw6gfx-get-mouse-state))
	   (mouse-x (assoc-ref mouse-state "x"))
	   (mouse-y (assoc-ref mouse-state "y"))
	   (coef (/ (lw6-config-get-number lw6def-mouse-sensitivity) %lw6-sensitivity-correction))
	   (x (hash-ref cursor "x"))
	   (y (hash-ref cursor "y"))
	   )
      (if (not (lw6-menu-has-items? (lw6-current-menu)))
	  (begin
	    (hash-set! cursor "x" (+ x (* (- mouse-x center-x) coef)))
	    (hash-set! cursor "y" (+ y (* (- mouse-y center-y) coef)))
	    (c-lw6gfx-warp-mouse center-x center-y)
	    ; return true if moved
	    (or (not (equal? mouse-x center-x))
		(not (equal? mouse-y center-y)))
	    )
	  #f
	  ))))

(define lw6-mover-keyboard-func
  (lw6-mover-generic c-lw6gfx-get-default-keys-state 
		     (lambda () (lw6-config-get-number lw6def-keyboard-sensitivity))
		     (lambda () (lw6-config-get-number lw6def-max-keyboard-cursor-speed))
		     ))

(define lw6-mover-joystick1-func
  (lw6-mover-generic (lambda () (let* (
				       (joystick-index (lw6-config-get-number lw6def-joystick1-index))
				       (joystick-button-a (lw6-config-get-number lw6def-joystick1-button-a))
				       (joystick-button-b (lw6-config-get-number lw6def-joystick1-button-b))
				       (state (c-lw6gfx-get-joystick-state joystick-index joystick-button-a joystick-button-b))
				       (x (assoc-ref state "x"))
				       (y (assoc-ref state "y"))
				       (button-a (assoc-ref state "button-a"))
				       (button-b (assoc-ref state "button-b"))
				       )
				  (list (cons "up" (< y 0))
					(cons "down" (> y 0))
					(cons "left" (< x 0))
					(cons "right" (>  x 0)))))
		     (lambda () (lw6-config-get-number lw6def-joystick-sensitivity))
		     (lambda () (lw6-config-get-number lw6def-max-joystick-cursor-speed))
		     ))

(define lw6-mover-joystick2-func
  (lw6-mover-generic (lambda () (let* (
				       (joystick-index (lw6-config-get-number lw6def-joystick2-index))
				       (joystick-button-a (lw6-config-get-number lw6def-joystick2-button-a))
				       (joystick-button-b (lw6-config-get-number lw6def-joystick2-button-b))
				       (state (c-lw6gfx-get-joystick-state joystick-index joystick-button-a joystick-button-b))
				       (x (assoc-ref state "x"))
				       (y (assoc-ref state "y"))
				       (button-a (assoc-ref state "button-a"))
				       (button-b (assoc-ref state "button-b"))
				       )
				  (list (cons "up" (< y 0))
					(cons "down" (> y 0))
					(cons "left" (< x 0))
					(cons "right" (>  x 0)))))
		     (lambda () (lw6-config-get-number lw6def-joystick-sensitivity))
		     (lambda () (lw6-config-get-number lw6def-max-joystick-cursor-speed))
		     ))

(define lw6-mover-bot-func
  (lambda (cursor)
    (let* (
	   (bot (hash-ref cursor "bot"))
	   (pos (if bot (c-lw6bot-next-move bot) #f))
	   )
      (if (and bot pos)
	  (begin
	    (hash-set! cursor "x" (assoc-ref pos "x"))
	    (hash-set! cursor "y" (assoc-ref pos "y"))
	    )))))

(define lw6-mover-universal-func
  (lambda (cursor)
    (begin
      ((lw6-mover-generic (lambda () 
			    (let* (
				   (key-state (c-lw6gfx-get-default-keys-state))
				   (key-up (assoc-ref key-state "up"))
				   (key-down (assoc-ref key-state "down"))
				   (key-left (assoc-ref key-state "left"))
				   (key-right (assoc-ref key-state "right"))
				   (joystick1-index (lw6-config-get-number lw6def-joystick1-index))
				   (joystick1-button-a (lw6-config-get-number lw6def-joystick1-button-a))
				   (joystick1-button-b (lw6-config-get-number lw6def-joystick1-button-b))
				   (joystick1-state (c-lw6gfx-get-joystick-state joystick1-index joystick1-button-a joystick1-button-b))
				   (joystick1-x (assoc-ref joystick1-state "x"))
				   (joystick1-y (assoc-ref joystick1-state "y"))
				   (joystick2-index (lw6-config-get-number lw6def-joystick2-index))
				   (joystick2-button-a (lw6-config-get-number lw6def-joystick2-button-a))
				   (joystick2-button-b (lw6-config-get-number lw6def-joystick2-button-b))
				   (joystick2-state (c-lw6gfx-get-joystick-state joystick2-index joystick2-button-a joystick2-button-b))
				   (joystick2-x (assoc-ref joystick2-state "x"))
				   (joystick2-y (assoc-ref joystick2-state "y"))
				   )
			      (list (cons "up" (or key-up 
						   (< joystick1-y 0)
						   (< joystick2-y 0)))
				    (cons "down" (or key-down
						     (> joystick1-y 0)
						     (> joystick2-y 0)))
				    (cons "left" (or key-left
						     (< joystick1-x 0)
						     (< joystick2-x 0)))
				    (cons "right" (or key-right
						      (> joystick1-x 0)
						      (> joystick2-x 0))))))  
			  (lambda () (lw6-config-get-number lw6def-keyboard-sensitivity))
			  (lambda () (lw6-config-get-number lw6def-max-keyboard-cursor-speed))
			  ) cursor)
      (lw6-mover-mouse-func cursor)
      )))

(define lw6-mover
  (lambda ()
    (map (lambda (cursor-key) (let* (
				     (game-state (lw6-get-game-global "game-state"))
				     (cursor (lw6-get-cursor cursor-key))
				     (mover (hash-ref cursor "mover"))
				     )
				(if (and game-state mover cursor)
				    (begin
				      (mover cursor)
				      (let* (					    
					     (coords (c-lw6pil-fix-coords game-state (hash-ref cursor "x") (hash-ref cursor "y") 0.0))
					     (x (assoc-ref coords "x"))
					     (y (assoc-ref coords "y"))
					     )
					(begin
					  (hash-set! cursor "x" x)
					  (hash-set! cursor "y" y)
					  ))
				      )
				    )))
	 (list "1" "2" "3" "4"))))

==> liquidwar6-0.0.10beta.tar.gz: liquidwar6-0.0.10beta/script/mover.scm <==
;Liquid War 6 is a unique multiplayer wargame.
;Copyright (C)  2005, 2006, 2007, 2008, 2009  Christian Mauduit <ufoot@ufoot.org>
;
;This program is free software; you can redistribute it and/or modify
;it under the terms of the GNU General Public License as published by
;the Free Software Foundation, either version 3 of the License, or
;
;(at your option) any later version.
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
;
;Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
;Contact author        : ufoot@ufoot.org

(define %lw6-sensitivity-correction 2.)

(define lw6-mover-slow-down
  (lambda (cursor sensitivity key)
    (let (
	  (value (hash-ref cursor key))
	  )
      (hash-set! cursor key (cond
			     ((> value 0)
			      (max 0 (- value (* 2.0 (/ sensitivity %lw6-sensitivity-correction)))))
			     ((< value 0)
			      (min 0 (+ value (* 2.0 (/ sensitivity %lw6-sensitivity-correction)))))
			     (#t 0)
			     )))))

(define lw6-mover-minus
  (lambda (cursor sensitivity max-cursor-speed key)
    (let (
	  (value (hash-ref cursor key))
	  )
      (hash-set! cursor key (cond
			     ((> value 0)
			      0)
			     (#t
			      (max (- 0 max-cursor-speed) (- value (/ sensitivity %lw6-sensitivity-correction))))
			     )))))

(define lw6-mover-plus
  (lambda (cursor sensitivity max-cursor-speed key)
    (let (
	  (value (hash-ref cursor key))
	  )
      (hash-set! cursor key (cond
			     ((< value 0)
			      0)
			     (#t
			      (min max-cursor-speed (+ value (/ sensitivity %lw6-sensitivity-correction))))
			     )))))

(define lw6-mover-generic
  (lambda (get-state-func sensitivity-func max-cursor-speed-func)
    (lambda (cursor)
      (let* (
	     (state (get-state-func))
	     (up (assoc-ref state "up"))
	     (down (assoc-ref state "down"))
	     (left (assoc-ref state "left"))
	     (right (assoc-ref state "right"))
	     (sensitivity (sensitivity-func))
	     (max-cursor-speed (max-cursor-speed-func))
	     (moved #f)
	    )
	(begin
	  (if up 
	      (if down
		  #t		  
		  (lw6-mover-minus cursor sensitivity max-cursor-speed "vy"))
	      (if down
		  (lw6-mover-plus cursor sensitivity max-cursor-speed "vy")
		  (lw6-mover-slow-down cursor sensitivity "vy")))
	  (if left
	      (if right
		  #t		  
		  (lw6-mover-minus cursor sensitivity max-cursor-speed "vx"))
	      (if right
		  (lw6-mover-plus cursor sensitivity max-cursor-speed "vx")
		  (lw6-mover-slow-down cursor sensitivity "vx")))
	  (if (or (not (= (hash-ref cursor "vx") 0))
		  (not (= (hash-ref cursor "vy") 0)))
	      (begin
		(hash-set! cursor "mouse-controlled" #f)
		(hash-set! cursor "x" (+ (hash-ref cursor "x") (hash-ref cursor "vx")))
		(hash-set! cursor "y" (+ (hash-ref cursor "y") (hash-ref cursor "vy")))
		#t)
	      #f
	      ))))))

(define lw6-mover-mouse-func
  (lambda (cursor)
    (let*
	(
	 (dsp (lw6-get-game-global "dsp"))
	 (mouse-state (c-lw6gui-mouse-poll-move dsp))
	 (map-x (assoc-ref mouse-state "map-x"))
	 (map-y (assoc-ref mouse-state "map-y"))
	 (menu-esc (assoc-ref mouse-state "menu-esc"))
	 )
      (if (or mouse-state (hash-ref cursor "mouse-controlled"))
	  (begin
	    (if (not mouse-state)
		(begin
		  (set! mouse-state (c-lw6gui-mouse-get-state dsp))
		  (set! map-x (assoc-ref mouse-state "map-x"))
		  (set! map-y (assoc-ref mouse-state "map-y"))
		  ))
	    (if (or (not (= (hash-ref cursor "x") map-x))
		    (not (= (hash-ref cursor "y") map-y)))
		(begin
		  (hash-set! cursor "x" map-x)
		  (hash-set! cursor "y" map-y)
		  (hash-set! cursor "mouse-controlled" #t)
		  #t)
		#f)
	    #t
	    )
	  #f
	  ))))

(define lw6-mover-keyboard-func
  (lw6-mover-generic (lambda () (c-lw6gui-keyboard-get-move-pad (lw6-get-game-global "dsp")))
		     (lambda () (lw6-config-get-number lw6def-cursor-sensitivity))
		     (lambda () (lw6-config-get-number lw6def-max-cursor-speed))
		     ))

(define lw6-mover-joystick1-func
  (lw6-mover-generic (lambda () (c-lw6gui-joystick1-get-move-pad (lw6-get-game-global "dsp")))
		     (lambda () (lw6-config-get-number lw6def-cursor-sensitivity))
		     (lambda () (lw6-config-get-number lw6def-max-cursor-speed))
		     ))

(define lw6-mover-joystick2-func
  (lw6-mover-generic (lambda () (c-lw6gui-joystick2-get-move-pad (lw6-get-game-global "dsp")))
		     (lambda () (lw6-config-get-number lw6def-cursor-sensitivity))
		     (lambda () (lw6-config-get-number lw6def-max-cursor-speed))
		     ))

(define lw6-mover-bot-func
  (lambda (cursor)
    (let* (
	   (bot (hash-ref cursor "bot"))
	   (pos (if bot (c-lw6bot-next-move bot) #f))
	   )
      (if (and bot pos)
	  (begin
	    (hash-set! cursor "x" (assoc-ref pos "x"))
	    (hash-set! cursor "y" (assoc-ref pos "y"))
	    ;;(tmp (list (assoc-ref pos "x") (assoc-ref pos "y")))
	    )))))

(define lw6-mover-universal-func
  (lambda (cursor)
    (begin
      ((lw6-mover-generic (lambda () 
			    (let* (
				   (dsp (lw6-get-game-global "dsp"))
				   (keyboard-state (c-lw6gui-keyboard-get-move-pad dsp))
				   (joystick1-state (c-lw6gui-joystick1-get-move-pad dsp))
				   (joystick2-state (c-lw6gui-joystick2-get-move-pad dsp))
				   )
			      (map (lambda (dir) 
				     (cons dir (or (assoc-ref keyboard-state dir)
						   (assoc-ref joystick1-state dir)
						   (assoc-ref joystick2-state dir))))
				   (list "up" "down" "left" "right"))
				))
			  (lambda () (lw6-config-get-number lw6def-cursor-sensitivity))
			  (lambda () (lw6-config-get-number lw6def-max-cursor-speed))
			  ) cursor)
      (lw6-mover-mouse-func cursor)
      )))

(define lw6-mover
  (lambda ()
    (map (lambda (cursor-key) (let* (
				     (game-state (lw6-get-game-global "game-state"))
				     (cursor (lw6-get-cursor cursor-key))
				     (mover (hash-ref cursor "mover"))
				     )
				(if (and game-state mover cursor (hash-ref cursor "x") (hash-ref cursor "y"))
				    (begin
				      (mover cursor)
				      (let* (					    
					     (coords (c-lw6pil-fix-coords game-state (hash-ref cursor "x") (hash-ref cursor "y") 0.0))
					     (x (assoc-ref coords "x"))
					     (y (assoc-ref coords "y"))
					     )
					(begin
					  (hash-set! cursor "x" x)
					  (hash-set! cursor "y" y)
					  ))
				      )
				    )))
	 (list "1" "2" "3" "4"))))
